/*
 * This file is part of the Configuration File Library (CFL).
 * Copyright (C) 2002-2003 Nicolas Darnis <ndarnis@free.fr>
 *
 * The CFL library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The CFL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the CFL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: cfl.h
 * $Revision: 1.11 $
 * $Date: 2003/06/03 13:11:45 $
 * $Author: ndarnis $
 * $Id: cfl.h,v 1.11 2003/06/03 13:11:45 ndarnis Exp $
 */

#ifndef _CFL_H_
#define _CFL_H_

#include <stdio.h>

#if defined (__cplusplus)
extern "C" {
#endif /* __cplusplus */

/* CFL error codes */
#define CFL_ERR_MALLOC             -1
#define CFL_ERR_SECTION_EXISTS     -2
#define CFL_ERR_SECTION_NOT_EXISTS -3
#define CFL_ERR_VALUE_EXISTS       -4
#define CFL_ERR_VALUE_NOT_EXISTS   -5
#define CFL_ERR_FILE_READ          -6
#define CFL_ERR_FILE_PARSE         -7

/**
 * cfl_value_type_t
 *
 * This is the list of available CFL values types.
 */
typedef enum 
{ 
  /** Boolean type (bool) */
  CFL_TYPE_BOOL = 0, 

  /** NULL terminated characters string type (char *) */
  CFL_TYPE_STRING, 

  /** Integer type (long) */
  CFL_TYPE_INTEGER,
 
  /** Float type (double) */
  CFL_TYPE_FLOAT 

} cfl_value_type_t;

/**
 * cfl_file_type_t
 *
 * This is the list of available CFL config files types.
 */
typedef enum
  {
    /** Default CFL file type (see documentation) */
    CFL_FILE_TYPE_DEFAULT = 0,

  } cfl_file_type_t;

/**
 * The CFL handle type.
 * This type is voluntary opaque. Variables of this kind could'nt be directly
 * used, but by the functions of this module.
 */
typedef struct _cfl_handle *cflh_t;

/**
 * Allocates a new cfl handle.
 * Creates a new cfl handle data structure with one built-in nameless section.
 * To use the built-in section, simply use NULL as its section name.
 * Returns the newly allocated handle in case of success.
 * Returns NULL in case of insufficient memory.
 *
 * Requires: nothing
 *
 * @see cfl_read()
 * @see cfl_free()
 */
extern cflh_t
cfl_alloc (void);

/**
 * Deallocates a cfl handle.
 * Deallocates and destroy the cfl HANDLE.
 *
 * Requires: HANDLE must be a valid cflh_t.
 *
 * @param HANDLE The cfl handle to deallocate
 * @see cfl_alloc()
 * @see cfl_read()
 */
extern void
cfl_free (cflh_t HANDLE                           /* The handle to deallocate */
	  );

/**
 * Allocates a new cfl handle and initializes it with a file.
 * Creates a new cfl handle data structure. Open the file INPUT_FILE_NAME to 
 * read it and initializes the new handle with the file's content. The file
 * must be of internal CFL format (please read documentation for more details).
 * Returns the new handle in case of success.
 * Returns NULL in case of failure.
 *
 * WARNING: . this function is *NOT* reentrant, so you should lock a mutex 
 *            before calling it and unlock the mutex after the call.
 *          . this function is *OBSOLETE* because it does not provide a way to
 *            know the error cause (it could be a memory allocation error,
 *            an impossibility to open the file, or an incorrect file format).
 *
 * Requires: nothing
 *
 * @param INPUT_FILE_NAME The name of the configuration file to read
 * @see cfl_alloc()
 * @see cfl_free()
 * @see cfl_init_from_file()
 * @see cfl_write()
 * @see cfl_write_xml()
 */
extern cflh_t
cfl_read (const char *INPUT_FILE_NAME  /* The name of the config file to read */
	  );

/**
 * Initializes a cfl handle with an opened file.
 * The cfl HANDLE is flushed before re-initializing it with the content of the
 * INPUT_FILE. The FILE_TYPE could be used to specify the type of the input
 * file (see definition of cfl_file_type_t above to view the supported types).
 * Returns 0 in case of success.
 * Returns CFL_ERR_MALLOC in case of insufficient memory.
 * Returns CFL_ERR_FILE_READ in case of INPUT_FILE is not readable.
 * Returns CFL_ERR_FILE_PARSE in case of incorrect format of INPUT_FILE.
 *
 * WARNING: this function is *NOT* reentrant, so maybe you'll need to lock a 
 *          mutex before calling it and unlock the mutex after the call.
 *
 * Requires: HANDLE must be a valid cflh_t & INPUT_FILE != NULL
 *
 * @param HANDLE The cfl handle to initialize
 * @param INPUT_FILE The configuration file to read
 * @param FILE_TYPE The type of the input file to read
 * @see cfl_alloc()
 * @see cfl_free()
 * @see cfl_read()
 * @see cfl_write()
 * @see cfl_write_xml()
 */
extern int
cfl_init_from_file (cflh_t HANDLE,                    /* Handle to initialize */
		    const FILE *INPUT_FILE,   /* File to read the config from */
		    const cfl_file_type_t FILE_TYPE       /* Type of the file */
		    );

/**
 * Write a cfl configuration handle to an opened file.
 * Writes the configuration contained in the cfl HANDLE to the file OUTPUT_FILE.
 *
 * Requires: HANDLE must be a valid cflh_t
 *           & OUTPUT_FILE must be a valid file
 *
 * @param HANDLE The configuration cfl handle to write
 * @param OUTPUT_FILE The file to write the configuration into
 * @see cfl_read()
 * @see cfl_init_from_file()
 * @see cfl_write_xml()
 */
extern void
cfl_write (const cflh_t HANDLE,                    /* The CFL handle to write */
	   FILE *OUTPUT_FILE                 /* The output file to write into */
	   );

/**
 * Writes a cfl configuration handle to an opened file into XML.
 * Writes the configuration contained in the cfl HANDLE to the file OUTPUT_FILE
 * into XML language.
 *
 * Requires: HANDLE must be a valid cfl handle
 *           & OUTPUT_FILE must be a valid file
 *
 * @param HANDLE The configuration cfl handle to write
 * @param OUTPUT_FILE The file to write the configuration into
 * @see cfl_read()
 * @see cfl_init_from_file()
 * @see cfl_write()
 */
extern void
cfl_write_xml (const cflh_t HANDLE,                    /* CFL handle to write */
	       FILE *OUTPUT_FILE                           /* The output file */
	       );

/**
 * Adds a new section into a cfl handle.
 * Adds a new section named SECTION_NAME into the cfl HANDLE.
 * Returns 0 in case of success.
 * Returns CFL_ERR_MALLOC in case of insufficient memory.
 * Returns CFL_ERR_SECTION_EXISTS if the SECTION_NAME already exists.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle in which the new section must go
 * @param SECTION_NAME The name of the new section to add
 * @see cfl_section_remove()
 */
extern int
cfl_section_insert (cflh_t HANDLE,     /* CFL handle to insert the section in */
		    const char *SECTION_NAME /* Name of the section to insert */
		    );

/**
 * Removes a section from a cfl handle.
 * Removes the section named SECTION_NAME with all its values from the cfl 
 * HANDLE.
 * Returns 0 in case of success.
 * Returns CFL_ERR_SECTION_NOT_EXISTS if the SECTION_NAME does not exists or
 * if SECTION_NAME == NULL (the built-in section could not be removed).
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to remove the section from
 * @param SECTION_NAME The name of the section to remove
 * @see cfl_section_insert()
 */
extern int
cfl_section_remove (cflh_t HANDLE,   /* CFL handle to remove the section from */
		    const char *SECTION_NAME /* Name of the section to remove */
		    );

/**
 * Gets the number of sections in a cfl handle.
 * Returns the number of sections contained in the cfl HANDLE.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to ciunt sections from
 * @see cfl_section_insert()
 * @see cfl_section_remove()
 * @see cfl_section_get_name_by_position()
 */
extern int
cfl_get_sections_count (cflh_t HANDLE    /* CFL handle to count sections from */
			);

/**
 * Gets the name of the section in a given position from a cfl handle.
 * Returns the section's name at the position POS of the cfl HANDLE.
 * Returns NULL if 1 >= POS > cfl_get_sections_count (HANDLE).
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle from which geting a section name
 * @param POS The position of the section to get the name
 * @see cfl_section_insert()
 * @see cfl_section_remove()
 * @see cfl_get_sections_count()
 */
extern char *
cfl_section_get_name_by_position (cflh_t HANDLE, 
				  unsigned long POS
				  );

/**
 * Gets the number of variables from a section.
 * Returns the number of variables into the SECTION_NAME of the HANDLE in case
 * of success.
 * Returns CFL_ERR_SECTION_NOT_EXISTS if the SECTION_NAME does not exists.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to use
 * @param SECTION_NAME The name of the section to get the number of values from
 * @see cfl_section_add()
 * @see cfl_section_remove()
 */
extern long
cfl_section_get_values_count (const cflh_t HANDLE,   /* The cfl handle to use */
			      const char *SECTION_NAME        /* Section name */
			      );

/**
 * Adds a new value into a section of a cfl handle.
 * Adds the new value named VALUE_NAME in the section SECTION_NAME into the cfl
 * HANDLE. The new value will be of type VALUE_TYPE and of value VALUE.
 * Returns 0 in case of success.
 * Returns CFL_ERR_MALLOC in case of insufficient memory.
 * Returns CFL_ERR_SECTION_NOT_EXISTS if the SECTION_NAME does not exists.
 * Returns CFL_ERR_VALUE_EXISTS if the VALUE_NAME already exists in the section.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle in which the new value must go
 * @param SECTION_NAME The name of the section in which the new value must go
 * @param VALUE_NAME The name of the value to insert
 * @param VALUE_TYPE The type of the value to insert
 * @param VALUE The value of the value to insert
 * @see cfl_value_remove()
 */
extern int
cfl_value_insert (cflh_t HANDLE,         /* CFL handle to insert a value into */
		  const char *SECTION_NAME,     /* Section to insert value in */
		  const char *VALUE_NAME,      /* Name of the value to insert */
		  const cfl_value_type_t VALUE_TYPE,            /* Value type */
		  void *VALUE                             /* The value itself */
		  );

/**
 * Remove a value from a section from a cfl handle.
 * Removes the value VALUE_NAME from the section SECTION_NAME of the cfl HANDLE.
 * Returns 0 in case of success.
 * Returns CFL_ERR_SECTION_NOT_EXISTS if the SECTION_NAME does not exists.
 * Returns CFL_ERR_VALUE_NOT_EXISTS if the VALUE_NAME does not exists in the
 * section.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to remove the value from
 * @param SECTION_NAME The name of the section in which the value is
 * @param VALUE_NAME The name of the value to remove
 * @see cfl_value_insert()
 */
extern int
cfl_value_remove (cflh_t HANDLE,       /* CFL handle to remove the value from */
		  const char *SECTION_NAME,     /* Section where the value is */
		  const char *VALUE_NAME       /* Name of the value to remove */
		  );

/**
 * Modifies a value of a cfl handle.
 * Changes the value VALUE_NAME of the section SECTION_NAME into the cfl HANDLE
 * with type VALUE_TYPE and value VALUE.
 * Returns 0 in case of success.
 * Returns CFL_ERR_MALLOC in case of insufficient memory.
 * Returns CFL_ERR_SECTION_NOT_EXISTS if the SECTION_NAME does not exists.
 * Returns CFL_ERR_VALUE_NOT_EXISTS if the VALUE_NAME does not exists.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle in which the value must be modified
 * @param SECTION_NAME The section in which the value is
 * @param VALUE_NAME The name of the value to modify
 * @param VALUE_TYPE The new type of the value
 * @param VALUE The new value of the value
 * @see cfl_value_get_by_name()
 * @see cfl_value_get_by_position()
 */
extern int
cfl_value_set_by_name (cflh_t HANLDE,    /* CFL handle to insert a value into */
		       const char *SECTION_NAME,/* Section to insert value in */
		       const char *VALUE_NAME, /* Name of the value to insert */
		       const cfl_value_type_t VALUE_TYPE,       /* Value type */
		       void *VALUE                        /* The value itself */
		       );

/**
 * Gets a value by its name from a section in cfl handle.
 * Returns the address of the VALUE_NAME from the SECTION_NAME in case of 
 * success. The type of the founded value is set to VALUE_TYPE.
 * Returns NULL if SECTION_NAME is not found or if VALUE_NAME is not found.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle from which getting the value
 * @param SECTION_NAME The name of the section in which the value is
 * @param VALUE_NAME The name of the value to get
 * @param VALUE_TYPE The address in which the type of the value will go
 * @see cfl_value_set_by_name()
 * @see cfl_value_get_by_position()
 */
extern void *
cfl_value_get_by_name (cflh_t HANDLE,     /* CFL handle to get the value from */
		       const char *SECTION_NAME,/* Section where the value is */
		       const char *VALUE_NAME,    /* Name of the value to get */
		       cfl_value_type_t *VALUE_TYPE             /* Value type */
		       );

/**
 * Gets a value by its position from a cfl handle.
 * Returns the address of the value at position POS from the section 
 * SECTION_NAME in the cfl HANDLE in case of success. The name of the founded 
 * value is set to VALUE_NAME and its type to VALUE_TYPE.
 * Returns NULL if SECTION_NAME is not found or if POS is incorrect.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle from which getting the value
 * @param SECTION_NAME The name of the section in which the value is
 * @param POS The position of the value to get
 * @param VALUE_NAME The address in which the name of the value will go
 * @param VALUE_TYPE The address in which the type of the value will go
 * @see cfl_value_get_by_name()
 * @see cfl_value_set_by_name()
 */
extern void *
cfl_value_get_by_position (const cflh_t HANDLE,   /* Handle to get value from */
			   const char *SECTION_NAME,  /* Section of the value */
			   unsigned long POS, /* Position of the value to get */
			   char **VALUE_NAME,                   /* Value name */
			   cfl_value_type_t *VALUE_TYPE         /* Value type */
			   );

/**
 * Gets the string corresponding to an error code.
 * Returns the string associated to the ERR_CODE.
 * Returns NULL if ERR_CODE is incorrect.
 *
 * Requires: nothing.
 *
 * @param ERR_CODE The CFL error code returned by a CFL function
 */
extern const char *
cfl_errcode_to_string (int ERR_CODE             /* CFL error code (see above) */
		       );

/**
 * Gets the error message corresponding to an error code.
 * Returns the message string associated to the ERR_CODE.
 * Returns NULL if ERR_CODE is incorrect.
 *
 * Requires: nothing.
 *
 * @param ERR_CODE The CFL error code returned by a CFL function
 */
extern const char *
cfl_errcode_to_message (int ERR_CODE            /* CFL error code (see above) */
			);

#ifdef __cplusplus
}
#endif/* __cplusplus */

#endif /* _CFL_H_ */
